<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"
  "http://www.w3.org/TR/html4/frameset.dtd">
<html{% if language is not none %} lang="{{ language }}"{% endif %}>
  <head>
    <title>{% trans version=version|e, docstitle=docstitle|e %}Changes in Version {{ version }} &#8212; {{ docstitle }}{% endtrans %}</title>
  </head>
  <frameset cols="45%,*">
    <frame name="main" src="changes.html">
    <frame name="src" src="about:blank">
  </frameset>
</html>
