Documentation.addTranslations({
    "locale": "zh_Hant_HK",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "",
        "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "",
        "&#169; Copyright %(copyright)s.": "",
        ", in ": "",
        "About these documents": "",
        "Automatically generated list of changes in version %(version)s": "",
        "C API changes": "",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "",
        "Collapse sidebar": "",
        "Complete Table of Contents": "",
        "Contents": "",
        "Copyright": "",
        "Created using <a href=\"https://www.sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "",
        "Expand sidebar": "",
        "Full index on one page": "",
        "General Index": "",
        "Global Module Index": "",
        "Go": "",
        "Hide Search Matches": "",
        "Index": "",
        "Index &ndash; %(key)s": "",
        "Index pages by letter": "",
        "Indices and tables:": "",
        "Last updated on %(last_updated)s.": "",
        "Library changes": "",
        "Navigation": "",
        "Next topic": "",
        "Other changes": "",
        "Overview": "",
        "Please activate JavaScript to enable the search\n    functionality.": "",
        "Preparing search...": "",
        "Previous topic": "",
        "Quick search": "",
        "Search": "",
        "Search Page": "",
        "Search Results": "",
        "Search finished, found ${resultCount} page(s) matching the search query.": "",
        "Search within %(docstitle)s": "",
        "Searching": "",
        "Searching for multiple words only shows matches that contain\n    all words.": "",
        "Show Source": "",
        "Table of Contents": "",
        "This Page": "",
        "Welcome! This is": "",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "",
        "all functions, classes, terms": "",
        "can be huge": "",
        "last updated": "",
        "lists all sections and subsections": "",
        "next chapter": "",
        "previous chapter": "",
        "quick access to all modules": "",
        "search": "",
        "search this documentation": "",
        "the documentation for": ""
    },
    "plural_expr": "0"
});